/*
Copyright (c) 2008, Michael Odenwald
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer 
   in the documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors may be used to endorse or promote products derived from
   this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdlib.h>
#include <inttypes.h>

#include "vfd.h"

/*
** vfd_puts
**
** send the string to the vfd_putc function
**
*/
void vfd_puts (char *s)
{
    while (*s) /* put string char by char to the send function */
    {   
        vfd_putc(*s);
        s++;
    }
}

/*
** vfd_puts_p
**
** send the string from program memory to the VFD
**
*/
void vfd_puts_p(const char *progmem_s)
{
    register char c;

    while((c = pgm_read_byte(progmem_s++)))
	{
		vfd_putc(c);
    }
}

/*
** vfd_version
**
** send the version string of the VFD library to the display
**
*/
void vfd_version(void)
{
	vfd_puts_P(VFD_VERSION);
}

/*
** vfd_euro
**
** send the code for the euro currency symbol to the vfd_putc function
**
*/
void vfd_euro(void)
{
	vfd_putc(0x1a);
}

/*
** vfd_linefeed
**
** send a linefeed to the vfd_putc function
**
*/
void vfd_linefeed(void)
{
	vfd_putc(0x0a);
}

/*
** vfd_hsm
**
** set the VFD to the 'Horizontal Scroll Mode'
**
*/
void vfd_hsm(void)
{
	vfd_putc(0x13);
}

/*
** vfd_bs
**
** send a backspace to the VFD
** 
** Input:
** flag for 'destructive' or 'none destructive' mode
**
*/
void vfd_bs(uint8_t des)
{
	vfd_putc(0x08);	
	if (des == VFD_BS_DESTRUCTIVE) /* destruktives Backspace ? */
	{
		vfd_putc(0x20);
		vfd_putc(0x08);
	}
}

/*
** vfd_intfont
**
** set the VFD to the 'International Font'
**
*/
void vfd_intfont(void)
{
	vfd_putc(0x18);
}

/*
** vfd_katafont
**
** set the VFD to the 'Katakana Font'
** This mode is usefull for bargraph o.e.
**
*/
void vfd_katafont(void)
{
	vfd_putc(0x19);
}

/*
** vfd_clsscr
**
** Send the Clear Display command and set the cursor at the position 1, 1
**
*/
void vfd_clrscr(void)
{
	vfd_putc(0x0e);
	vfd_putc(0x1b);
	vfd_putc(0x48);
	vfd_putc(0);
}

/*
** vfd_cursor_on
**
** Switch the underline cursor on
**
*/
void vfd_cursor_on(void)
{
	vfd_putc(0x14);
}

/*
** vfd_cursor_on_blinking
**
** Switch on a blinking underline cursor
**
*/
void vfd_cursor_on_blinking(void)
{
	vfd_putc(0x17);
}

/*
** vfd_cursor_on_char
**
** Switch on a blinking char cursor
**
*/
void vfd_cursor_on_char(void)
{
	vfd_putc(0x15);
}

/*
** vfd_cursor_off
**
** Switch the cursor off (invisible)
**
*/
void vfd_cursor_off(void)
{
	vfd_putc(0x16);
}

/*
** vfd_gotoxy
**
** Set the next write position of a char to the position x, y
**
** Input:
** 1 < x < VFD_COLUMNS
** 1 < y < VFD_ROWS
**
** Output:
** 0 no error
** 1 parameter error	
**
*/
int vfd_gotoxy(uint8_t x, uint8_t y)
{
	if (x >	VFD_COLUMNS) return 1;
	if (y > VFD_ROWS) return 1;

	vfd_putc(0x1b);
	vfd_putc(0x48);

	vfd_putc(x - 1 + ((y - 1) * VFD_COLUMNS));
	return 0;
}

/*
** vfd_init
**
** Initialize the VFD and set it to de defined status
**
** Init do: clear display
**          cursor to position 1, 1
**          cursor on
**          international font on
** Input:
** the callback function for the low level output function, of one character to the VFD
**
** Output:
** 0 no error
**
*/
int vfd_init(int (*f) (uint8_t c))
{
	vfd_putc = f;
	vfd_intfont();
	vfd_clrscr();
	vfd_gotoxy(1,1);
	vfd_cursor_on();
	return 0;
}
